@@ -1,4 +1,4 @@
-;; Copyright 2001, 2002 Matthias Koeppe <mkoeppe@mail.math.uni-magdeburg.de>
+;; Copyright 2001, 2002, 2007 Matthias Koeppe <mkoeppe@mail.math.uni-magdeburg.de>
 ;;
 ;; This code is free software; you can redistribute it and/or
 ;; modify it under the terms of version 2.1 of the GNU Lesser 
@@ -9,23 +9,13 @@
 ;; with names and copyright holders altered accordingly.
 
 
-(in-package :common-lisp-user)
-
-;;(let ((*load-verbose* nil)
-;;      (*compile-verbose* nil))
-;;  (load "/home/mkoeppe/p/cl-bibtex/bibtex.system")
-;;  (mk:operate-on-system :bibtex 'compile)
-;;  (mk:operate-on-system :bibtex 'load))
-
-(setq *load-verbose* nil)
-
-(require :bibtex)
+(in-package :bibtex-program)
 
 ;;; The emulation of the bibtex program
 
 (defun do-emulate-bibtex (command-line-args)
   (let ((*min-crossrefs* 2)
-	(*bibtex-split-initials* nil)
+	(*bibtex-split-initials* t)
 	(file-stem nil))
     (dolist (arg command-line-args)
       (cond
@@ -33,6 +23,8 @@
 	     (length "-min-crossrefs="))
 	(setq *min-crossrefs*
 	      (parse-integer arg :start (length "-min-crossrefs="))))
+       ((string= arg "-no-split-initials")
+	(setq *bibtex-split-initials* nil))
        ((string= arg "-split-initials")
 	(setq *bibtex-split-initials* t))
        ((string= arg "-terse")
@@ -47,16 +39,36 @@
     (unless file-stem
       (error "Need exactly one file argument"))
     (format *error-output*
-	    "This is CL-BibTeX, Version ~A" bibtex-compiler::+version+)
+	    "This is CL-BibTeX, Version ~A~%" bibtex-compiler::+version+)
     (bibtex-compiler:bibtex file-stem)))
 
+(defun quit (&optional code)
+  #+abcl (ext:quit code)
+  #+allegro (excl:exit code)
+  #+clisp (#+lisp=cl ext:quit #-lisp=cl lisp:quit code)
+  #+cmu (ext:quit code)
+  #+cormanlisp (win32:exitprocess code)
+  #+gcl (lisp:bye code)
+  #+lispworks (lw:quit :status code)
+  #+lucid (lcl:quit code)
+  #+sbcl (sb-ext:quit :unix-status
+                      (typecase code ((signed-byte 32) code) (null 0) (t 1)))
+  #-(or allegro clisp cmu cormanlisp gcl lispworks lucid sbcl)
+  (error 'not-implemented :proc (list 'quit code)))
+
 (defun emulate-bibtex (argv)
   ;;(princ "bar") (terpri)
-  (let ((*gc-verbose* nil))
-    (handler-case (do-emulate-bibtex argv)
-      (error (condition)
-	(format *error-output* "~&bibtex: ~A~%"
-		condition)
-	(format *error-output* "~&Try `bibtex --help' for more information.~%")
-	(unix:unix-exit 1)))))
+  (let (#+cmu (*gc-verbose* nil))
+    (multiple-value-bind (history err-count)
+	(handler-case (do-emulate-bibtex argv)
+	  (error (condition)
+	    (format *error-output* "~&bibtex: ~A~%"
+		    condition)
+	    (format *error-output* "~&Try `bibtex --help' for more information.~%")
+	    (quit 4)))
+      (unless (zerop history)
+	(format *error-output* 
+		"~&(There ~[were~;was~:;were~] ~:*~D ~[~;warning~;error~;fatal~] message~:*~:P)~%"
+		err-count history))
+      (quit history))))
 
