@@ -62,6 +62,20 @@ everything up to the beginning of the next entry."
   (mark-fatal)
   (error "Fatal BibTeX error"))
 
+;;; Interface to BibTeX entries
+
+;; BibTeX entries are EQUALP hash tables, the keys being arbitrary strings.
+;; In addition, we use two symbols as keys:
+
+(defvar +cite-key+ '+cite-key+)
+(defvar +entry-type+ '+entry-type+)
+
+(defmacro bib-entry-cite-key (entry)
+  `(gethash +cite-key+ ,entry))
+
+(defmacro bib-entry-type (entry)
+  `(gethash +entry-type+ ,entry))
+
 ;;; Reading the database files
 
 (defvar *bib-stream* nil)
@@ -73,6 +87,8 @@ everything up to the beginning of the next entry."
 (defvar *bib-entry-type-functions* nil
   "An alist mapping BibTeX entry types to formatter functions")
 
+(defparameter *identifier-growth* 32)
+
 (defun read-bib-identifier ()
   "Read an identifier from *BIB-STREAM*, returning it as a string, or
 nil if no identifier could be read."
@@ -82,7 +98,7 @@ nil if no identifier could be read."
 	(loop as char = (peek-char nil *bib-stream*)
 	      until (or (whitespace-p char)
 			(member char '(#\" #\# #\% #\' #\( #\) #\, #\= #\{ #\} )))
-	      do (vector-push-extend (read-char *bib-stream*) s))
+	      do (vector-push-extend (read-char *bib-stream*) s *identifier-growth*))
 	(if (zerop (length s))
 	    nil
 	    s))))
@@ -113,19 +129,19 @@ hash-table *BIB-DATABASE* and using/updating the macro hash-table
 pairs of braces. Return a string of everything read, except for the
 right delimiter."
   (let ((brace-level 0)
-	(chars '()))
+	(result (make-array 32 :element-type 'character :fill-pointer 0 :adjustable t)))
     (loop (let ((char (read-char stream nil nil)))
 	    (cond
 	      ((and (char= char right-delimiter)
 		    (zerop brace-level))
-	       (return (coerce (reverse chars) 'string)))
+	       (return result))
 	      ((char= char #\{)
 	       (incf brace-level))
 	      ((char= char #\})
 	       (if (zerop brace-level)
 		   (bib-warn "Unbalanced braces")
 		   (decf brace-level))))
-	    (setq chars (cons char chars))))))
+	    (vector-push-extend char result 32)))))
 
 (defun read-bib-field-token ()
   (case (peek-char t *bib-stream*)
@@ -152,10 +168,10 @@ right delimiter."
 	   (t
 	    definition)))))))
 
-(defconstant +bib-whitespace-character-list+
+(defvar +bib-whitespace-character-list+
   '(#\Newline #\Space #\Linefeed #\Return #\Page #\Tab))
 
-(defconstant +bib-sep-character-list+
+(defvar +bib-sep-character-list+
   '(#\~ #\-))
 
 (defun whitespace-p (char)
@@ -257,12 +273,14 @@ non-nil, remove any leading or trailing whitespace."
 	  collecting char into list
 	  finally (if (char= char #\,) (unread-char char *bib-stream*))
 	  (setq key (coerce list 'string)))
+    (when (gethash key *bib-database*) 
+      (bib-error "Repeated entry---~A" key))
     (unless (char= (peek-char t *bib-stream*) #\,)
       (bib-error "Expected `,' character"))
     (read-char *bib-stream*)
     (let ((entry (make-bib-entry)))
-      (setf (gethash "entry-type" entry) (string-downcase entry-type))
-      (setf (gethash "key" entry) key) 
+      (setf (bib-entry-type entry) (string-downcase entry-type))
+      (setf (bib-entry-cite-key entry) key) 
       (loop (multiple-value-bind (name value)
 		(read-bib-field t)
 	      (setf (gethash name entry)
@@ -284,12 +302,12 @@ non-nil, remove any leading or trailing whitespace."
 
 (defun write-bib-entry (entry &optional (stream *standard-output*))
   (format stream "~&@~A{~A"
-	  (gethash "entry-type" entry)
-	  (gethash "key" entry))
+	  (bib-entry-type entry)
+	  (bib-entry-cite-key entry))
   (loop for field being each hash-key in entry
 	and value being each hash-value in entry
-	unless (or (string-equal field "entry-type")
-		   (string-equal field "key"))
+	unless (or (eql field +entry-type+)
+		   (eql field +cite-key+))
 	do (format stream ",~%  ~A = {~A}" field value))
   (format stream "~%}~%"))
 
@@ -350,7 +368,7 @@ well."
     (cond
       ((eql cite-keys t)
        (loop for key being each hash-key in *bib-database*
-	     do (vector-push-extend (get-merged-bib-entry key) bib-entries)))
+	     do (vector-push-extend (get-merged-bib-entry key) bib-entries 256)))
       (t
        (let ((crossref-hash (make-hash-table :test 'equalp))
 	     (processed-keys '()))
@@ -373,13 +391,13 @@ well."
 	   (dolist (key cite-keys)
 	     (let ((entry (get-merged-bib-entry key)))
 	       (when entry
-		 (vector-push-extend entry bib-entries))))
+		 (vector-push-extend entry bib-entries 256))))
 	   (loop for key being each hash-key in crossref-hash
 		 and count being each hash-value in crossref-hash
 		 when (>= count min-crossrefs)
 		 do (let ((entry (get-merged-bib-entry key)))
 		      (when entry
-			(vector-push-extend entry bib-entries)))))))
+			(vector-push-extend entry bib-entries 256)))))))
     (coerce bib-entries 'list)))
 
 ;;; BibTeX names
@@ -389,13 +407,14 @@ well."
 list of name tokens (SEP-CHAR TOKEN-STRING)."
   first von last jr)
 
-(defvar *bibtex-split-initials* nil
+(defvar *bibtex-split-initials* t
   "If non-nil, BibTeX understands that there are two abbreviated first
 names in names like `Padberg, M.W.'.  The original BibTeX 0.99c,
 written by Oren Patashnik in the WEB language, thinks that `M.W.' is a
 single first name; hence, in abbreviated format, the name becomes
-`M. Padberg' instead of `M. W. Padberg'.  That's the reason why the
-default value of this variable is nil.")
+`M. Padberg' instead of `M. W. Padberg'.")
+
+(defvar *bibtex-split-initials-already-warned-hashtable* nil)
 
 (defun tokenize-bibtex-name (name-string &key (start 0) (end nil))
   "Break a BibTeX name into name tokens."
@@ -455,7 +474,14 @@ default value of this variable is nil.")
 	  ((and (char= char #\.)
 		*bibtex-split-initials*
 		(zerop brace-level)
-		token-start)
+		token-start
+		(< (1+ index) end)
+		(let ((next-char (char name-string (1+ index))))
+		  (and (not (whitespace-p next-char)) (not (sepchar-p next-char)))))
+	   (let ((name (subseq name-string start (or end (length name-string)))))
+	     (unless (gethash name *bibtex-split-initials-already-warned-hashtable*)
+	       (bib-warn "Splitting the initials in the name `~A';~%  I suggest you add spaces between initials in the database entry" name)
+	       (setf (gethash name  *bibtex-split-initials-already-warned-hashtable*) t)))
 	   (setq tokens (cons (cons (or sep-char #\Space)
 				    (subseq name-string token-start (+ index 1)))
 			      tokens)
@@ -476,10 +502,58 @@ default value of this variable is nil.")
 	       (setq token-start index))))))))
 
 (defun von-token-p (token)
-  ;; FIXME: The original BibTeX code also checks whether control
-  ;; sequences leading to lower-case characters are used.
-  (and (> (length (cdr token)) 0)
-       (lower-case-p (char (cdr token) 0))))
+  ;; From the BibTeX source code: "the von name
+  ;; starts with the first nonlast token whose first brace-level-0 letter
+  ;; is in lower case (for the purposes of this determination, an accented
+  ;; or foreign character at brace-level-1 that's in lower case will do, as
+  ;; well)."
+  (let ((string (cdr token)))
+    (and (> (length string) 0)
+	 (with-input-from-string (s string)
+	   (let ((brace-level 0))
+	     (loop 
+		for char = (read-char s nil nil)
+		while char
+		do (cond ((char= char #\{) 
+			   (incf brace-level)
+			   (when (and (= brace-level 1)
+				      (char= (peek-char nil s nil #\Space) #\\))
+			     ;; @<Check the special character (and |return|)@>=
+			     (read-char s) ;consume backslash
+			     ;; Check whether it is a special character; otherwise, 
+			     ;; just ignore the control sequence.
+			     (let* ((control-sequence (read-tex-control-sequence s))
+				    (foreign-character (find-foreign-character control-sequence)))
+			       (when foreign-character
+				 (return (lower-case-p (char control-sequence 0)))))
+			     ;; It is not a special character, so
+			     ;; treat it as an accented character.  We
+			     ;; now only scan till the end of the
+			     ;; brace group; the first alphabetic
+			     ;; character decides.  If nothing is
+			     ;; found within the brace group, decide
+			     ;; that is not a "von" token.
+			     (loop 
+				for char = (read-char s nil nil)
+				while char 
+				while (plusp brace-level)
+				do (cond ((upper-case-p char) 
+					  (return-from von-token-p nil))
+					 ((lower-case-p char) 
+					  (return-from von-token-p t))
+					 ((char= char #\{)
+					  (incf brace-level))
+					 ((char= char #\})
+					  (decf brace-level)))
+				finally (return-from von-token-p nil))))
+			 ((plusp brace-level)
+			  (when (char= char #\}) 
+			    (decf brace-level)))
+			 ;; The first alphabetic char at brace-level 0 decides:
+			 ((upper-case-p char) 
+			  (return nil))
+			 ((lower-case-p char) 
+			  (return t)))))))))
 
 (defun parse-bibtex-name (name-string &key (start 0) (end nil))
   "Break a BibTeX name into its components, returning a BIBTEX-NAME
@@ -608,7 +682,8 @@ many text characters.")
   (do ((tokens tokens (cdr tokens))
        (token-index 0 (+ token-index 1)))
       ((null tokens))
-    (let ((token (car tokens)))
+    (let ((token (car tokens))
+	  (*print-pretty* nil))
       (unless (zerop token-index)
 	(cond
 	  (inter-token-string
@@ -628,11 +703,13 @@ many text characters.")
       (cond
 	(full
 	 (princ (cdr token) stream))
-	(t ;; FIXME: Handle special characters here (415)
-	 (princ (char (cdr token) 0) stream))))))
+	(t ;; Abbreviate
+	 (write-tex-group (list (first (parse-tex-string (cdr token))))
+			  stream))))))
 
 (defun format-bibtex-name (stream format-string bibtex-name)
-  (let ((string-output nil)
+  (let ((*print-pretty* nil)
+	(string-output nil)
 	(end (length format-string)))
     (cond
       ((null stream) (setq stream (make-string-output-stream)
@@ -784,7 +861,7 @@ trailing whitespace is flushed if :SKIP-WHITESPACE is non-nil."
 				  :initial-element char)))
 	  (loop for char = (peek-char nil stream nil #\Space)
 		while (tex-alpha-char-p char :at-is-letter at-is-letter)
-		do (vector-push-extend (read-char stream) result))
+		do (vector-push-extend (read-char stream) result 32))
 	  (when skip-whitespace
 	    (peek-char t stream nil nil))
 	  result)
@@ -808,6 +885,8 @@ found in *AUX-FILE-COMMANDS*, call the associated function."
 	  (progn
 	    (format *error-output* "~&I'm skipping whatever remains of this command~%")
 	    (read-line *aux-stream* nil ""))))))
+
+(defvar *aux-file-level* 0)
 	   
 (defun read-aux-file-recursively (name)
   (let ((full-name (kpathsea:find-file name)))
@@ -816,6 +895,9 @@ found in *AUX-FILE-COMMANDS*, call the associated function."
     (with-open-file (*aux-stream* full-name :if-does-not-exist nil)
       (unless *aux-stream*
 	(bib-fatal "I couldn't open auxiliary file: ~S" name))
+      (if (zerop *aux-file-level*)
+	  (format *error-output* "~&The top-level auxiliary file: ~A~%" full-name)
+	  (format *error-output* "~&A level-~D auxiliary file: ~A~%" *aux-file-level* full-name))
       (loop as char = (peek-char nil *aux-stream* nil nil)
 	    while char
 	    do (cond
@@ -828,7 +910,8 @@ found in *AUX-FILE-COMMANDS*, call the associated function."
   "Read an AUX file, modifying *CITE-KEYS*, *CITE-ALL-ENTRIES*,
 *BIB-FILES*, and *BIB-STYLE*."
   (let ((*citation-seen-p* nil)
-	(*bibdata-seen-p* nil))
+	(*bibdata-seen-p* nil)
+	(*aux-file-level* 0))
     (read-aux-file-recursively name)
   ;; check everything ok
   (flet ((aux-end-error (what)
@@ -847,7 +930,8 @@ found in *AUX-FILE-COMMANDS*, call the associated function."
   "Process an AUX-file \\@input command."
   (unless (char= (read-char *aux-stream* nil #\Space) #\{)
     (aux-error "Expected `{'"))
-  (let ((file-name (scan-balanced-braces *aux-stream* #\})))
+  (let ((file-name (scan-balanced-braces *aux-stream* #\}))
+	(*aux-file-level* (1+ *aux-file-level*)))
     (read-aux-file-recursively file-name)))
 
 (defun aux-bibstyle-command ()
@@ -867,7 +951,7 @@ the delimiter, which is left in the stream."
     (loop as char = (peek-char nil stream nil nil)
 	  until (or (null char)
 		    (member char delimiters))
-	  do (vector-push-extend (read-char stream) s))
+	  do (vector-push-extend (read-char stream) s 64))
     s))
 
 (defun aux-bibdata-command ()
@@ -922,41 +1006,42 @@ output line.  If there's no whitespace character to break the line at,
 we break it just before *BBL-MAX-PRINT-LINE*, append a comment
 character (%), and don't indent the next line."
   (loop for char across string do
- 	(vector-push-extend char *bbl-line-buffer*))
+       (vector-push-extend char *bbl-line-buffer* 128))
   (when *bbl-max-print-line*
-    (loop while (> (length *bbl-line-buffer*) *bbl-max-print-line*) do
-	  ;; Break that line
-	  (let ((white-space-index
-		 (position-if #'whitespace-p *bbl-line-buffer*
-			      :start *bbl-min-print-line*
-			      :end (+ *bbl-max-print-line* 1)
-			      :from-end t)))
-	    (cond
-	      (white-space-index	; have a whitespace character
-	       (let ((non-space-index
-		      (position-if (complement #'whitespace-p) *bbl-line-buffer*
-				   :end white-space-index
-				   :from-end t)))
-		 (when non-space-index ; ignore a line of just whitespace
-		   (princ (subseq *bbl-line-buffer* 0 (+ non-space-index 1))
-			  *bbl-output*)
-		   (terpri *bbl-output*)
-		   ;; shift the rest back
-		   (replace *bbl-line-buffer* *bbl-line-buffer*
-			    :start1 2 :start2 (+ white-space-index 1))
-		   ;; start the next line with two spaces
-		   (replace *bbl-line-buffer* "  " :start1 0)
-		   (decf (fill-pointer *bbl-line-buffer*)
-			 (- (+ white-space-index 1) 2)))))
-	      (t ; there's no whitespace character to break the line at
-	       (princ (subseq *bbl-line-buffer* 0 (- *bbl-max-print-line* 1))
-		      *bbl-output*)
-	       (princ "%" *bbl-output*)
-	       (terpri *bbl-output*)
-	       (replace *bbl-line-buffer* *bbl-line-buffer*
-			:start1 0 :start2 (- *bbl-max-print-line* 1))
-	       (decf (fill-pointer *bbl-line-buffer*)
-		     (- *bbl-max-print-line* 1))))))))
+    (let ((*print-pretty* nil))
+      (loop while (> (length *bbl-line-buffer*) *bbl-max-print-line*) do
+	 ;; Break that line
+	   (let ((white-space-index
+		  (position-if #'whitespace-p *bbl-line-buffer*
+			       :start *bbl-min-print-line*
+			       :end (+ *bbl-max-print-line* 1)
+			       :from-end t)))
+	     (cond
+	       (white-space-index	; have a whitespace character
+		(let ((non-space-index
+		       (position-if (complement #'whitespace-p) *bbl-line-buffer*
+				    :end white-space-index
+				    :from-end t)))
+		  (when non-space-index	; ignore a line of just whitespace
+		    (princ (subseq *bbl-line-buffer* 0 (+ non-space-index 1))
+			   *bbl-output*)
+		    (terpri *bbl-output*)
+		    ;; shift the rest back
+		    (replace *bbl-line-buffer* *bbl-line-buffer*
+			     :start1 2 :start2 (+ white-space-index 1))
+		    ;; start the next line with two spaces
+		    (replace *bbl-line-buffer* "  " :start1 0)
+		    (decf (fill-pointer *bbl-line-buffer*)
+			  (- (+ white-space-index 1) 2)))))
+	       (t ; there's no whitespace character to break the line at
+		(princ (subseq *bbl-line-buffer* 0 (- *bbl-max-print-line* 1))
+		       *bbl-output*)
+		(princ "%" *bbl-output*)
+		(terpri *bbl-output*)
+		(replace *bbl-line-buffer* *bbl-line-buffer*
+			 :start1 0 :start2 (- *bbl-max-print-line* 1))
+		(decf (fill-pointer *bbl-line-buffer*)
+		      (- *bbl-max-print-line* 1)))))))))
 
 (defun bbl-terpri ()
   ;; Trim trailing whitespace
@@ -964,8 +1049,9 @@ character (%), and don't indent the next line."
 		   (whitespace-p (char *bbl-line-buffer*
 					(- (length *bbl-line-buffer*) 1))))
 	do (decf (fill-pointer *bbl-line-buffer*)))
-  (princ *bbl-line-buffer* *bbl-output*)
-  (terpri *bbl-output*)
+  (let ((*print-pretty* nil))
+    (princ *bbl-line-buffer* *bbl-output*)
+    (terpri *bbl-output*))
   (setf (fill-pointer *bbl-line-buffer*) 0)
   nil)
 
@@ -988,17 +1074,88 @@ character (%), and don't indent the next line."
 least *MIN-CROSSREFS* times, it is included as a separate entry as
 well.")
 
+(defun parse-equivalent-entries-string (string)
+  (mapcar (lambda (s)
+	    (string-trim +bib-whitespace-character-list+ s))
+	  (split-sequence:split-sequence #\, string)))
+;; (parse-equivalent-entries-string "newbar,barvinok-2006-ehrhart-quasipolynomial")
+;; (parse-equivalent-entries-string "  newbar ,  barvinok-2006-ehrhart-quasipolynomial ")
+
+(defun compute-bib-equivalence-classes ()
+  "Return a list of lists of equivalent keys."
+  (let ((marks (make-hash-table :test 'equalp))
+	(components nil)
+	(neighbors-hash (make-hash-table :test 'equalp)))
+    ;; Compute the neighbors (symmetric)
+    (loop for anchor being each hash-key in *bib-database* using (hash-value entry)
+       do (let* ((keys-string (gethash "EQUIVALENT-ENTRIES" entry))
+		 (new-equivalent-keys (and keys-string
+					   (parse-equivalent-entries-string keys-string))))
+;; 	    (when new-equivalent-keys
+;; 	      (format *error-output* "~&;; Equivalent to ~A:~{ ~A~}~%" anchor new-equivalent-keys))
+	    (setf (gethash anchor neighbors-hash) 
+		  (union new-equivalent-keys (gethash anchor neighbors-hash)))
+	    (dolist (key new-equivalent-keys)
+	      (pushnew anchor (gethash key neighbors-hash) :test 'equalp))))
+    (loop for anchor being each hash-key in neighbors-hash using (hash-value neighbors)
+       unless (gethash anchor marks) 
+       do (let ((component (list anchor)))
+	    (labels ((visit (node predecessors)
+		       (setf (gethash node marks) t)
+		       (dolist (neighbor-node neighbors)
+			 (cond
+			   ((gethash neighbor-node marks)) ; already marked
+			   (t
+			    (push neighbor-node component)
+			    (visit neighbor-node (cons node predecessors)))))))
+	      (visit anchor nil)
+	      (push component components))))
+    components))
+;; (let ((*bib-database* (make-hash-table :test 'equalp)) (*bib-macros* (make-hash-table :test #'equalp)) (*bib-files* '("iba-bib" "weismant" "barvinok"))) (read-all-bib-files) (break) (compute-bib-equivalence-classes))
+
+(defun read-all-bib-files ()
+  (loop for file in *bib-files*
+     for file-number from 1
+     do (let ((expanded-file (kpathsea:find-file (concatenate 'string file ".bib"))))
+	  (if (not expanded-file)
+	      (format *error-output* "I couldn't find database file `~A.bib'" file)
+	      (with-open-file (s expanded-file :if-does-not-exist nil)
+		(if (not s)
+		    (format *error-output* "I couldn't open database file `~A.bib'" expanded-file)
+		    (progn
+		      (format *error-output* "~&Database file #~D: ~A~%" file-number expanded-file)
+		      (read-bib-database s))))))))
+
+(defparameter *check-multiple-cited-equivalent-entries* t
+  "If true, perform the following extension of BibTeX behavior:
+The attribute EQUIVALENT-ENTRIES of a bibliographic entry
+specifies the keys of other bibliographic entries that are to be
+considered equivalent.  We compute the equivalence classes
+spanned by these equivalences, and warn if multiple equivalent
+keys are used in one document (we warn because both we and the
+original BibTeX will produce a bibliography that contains several
+copies of that entry).")
+
+(defun check-multiple-cited-equivalent-entries (bib-entries)
+  (let ((equivalence-classes
+	 (compute-bib-equivalence-classes))
+	(bib-keys (mapcar (lambda (x) (bib-entry-cite-key x)) bib-entries)))
+    (loop for class in equivalence-classes
+       do (let ((cited-equivalent-keys
+		 (remove-if-not (lambda (key) (member key bib-keys :test 'equalp))
+				class)))
+	    (when (> (length cited-equivalent-keys) 1)
+	      (bib-warn "These equivalent entries have been used:~{ ~A~}" 
+			cited-equivalent-keys))))))
+
 (defun read-all-bib-files-and-compute-bib-entries ()
-  (dolist (file *bib-files*)
-    (let ((expanded-file (kpathsea:find-file (concatenate 'string file ".bib"))))
-      (if (not expanded-file)
-	  (format *error-output* "I couldn't find database file `~A.bib'" file)
-	  (with-open-file (s expanded-file :if-does-not-exist nil)
-	    (if (not s)
-		(format *error-output* "I couldn't open database file `~A.bib'" expanded-file)
-		(read-bib-database s))))))
-  (cited-bib-entries (if *cite-all-entries* t *cite-keys*)
-		     :min-crossrefs *min-crossrefs*))
+  (read-all-bib-files)
+  (let ((bib-entries
+	 (cited-bib-entries (if *cite-all-entries* t *cite-keys*)
+			    :min-crossrefs *min-crossrefs*)))
+    (when *check-multiple-cited-equivalent-entries*
+      (check-multiple-cited-equivalent-entries bib-entries))
+    bib-entries))
 
 ;;; Misc functions
 
@@ -1055,23 +1212,24 @@ control sequences or sub-lists representing groups."
   "Write the TeX GROUP to STREAM.  If a control word occurs at the end
 of the group, terminate it with whitespace unless NO-TERMINATE-P is
 true."
-  (loop for (token . tail) on group
-	do (etypecase token
-	     (character (princ token stream))
-	     (list (princ "{" stream)
-		   (write-tex-group token stream t)
-		   (princ "}" stream))
-	     (string (princ "\\" stream)
-		     (princ token stream)
-		     (unless (and (= (length token) 1)
-				  (not (alpha-char-p (char token 0))))
-		       ;; Control word, must be terminated
-		       (unless no-terminate-p ; unless otherwise requested
-			 (when (or (null tail)
-				   (and (characterp (car tail))
-					(alpha-char-p (car tail))))
-			   ; when a character or nothing (unknown stuff) follows
-			   (princ " " stream))))))))
+  (let ((*print-pretty* nil))
+    (loop for (token . tail) on group
+       do (etypecase token
+	    (character (princ token stream))
+	    (list (princ "{" stream)
+		  (write-tex-group token stream t)
+		  (princ "}" stream))
+	    (string (princ "\\" stream)
+		    (princ token stream)
+		    (unless (and (= (length token) 1)
+				 (not (alpha-char-p (char token 0))))
+		      ;; Control word, must be terminated
+		      (unless no-terminate-p ; unless otherwise requested
+			(when (or (null tail)
+				  (and (characterp (car tail))
+				       (alpha-char-p (car tail))))
+					; when a character or nothing (unknown stuff) follows
+			  (princ " " stream)))))))))
 
 (defun for-all-tex-tokens (function string-or-group)
   "Call FUNCTION for every token in the given TeX string (a string or
@@ -1310,7 +1468,7 @@ special characters in a very complicated way."
 
 ;;; The approximate calculation of text widths
 
-(defconstant +cmr10-character-widths+
+(defvar +cmr10-character-widths+
   '((#\Space . 278) (#\! . 278) (#\" . 500) (#\# . 833) (#\$ . 500) (#\% . 833)
     (#\& . 778) (#\' . 278) (#\( . 389) (#\) . 389) (#\* . 500) (#\+ . 778)
     (#\, . 278) (#\- . 333) (#\. . 278) (#\/ . 500) (#\0 . 500) (#\1 . 500)
@@ -1391,4 +1549,30 @@ specially."
 
 (parse-bibtex-name-list '())
 
+(format-bibtex-name t "{f.} {l.}" (parse-bibtex-name "Matiyasevich, {\\relax{Yu}}ri V."))
+(format-bibtex-name t "{f.} {l.}" (parse-bibtex-name "Matiyasevich, {\\'E}"))
+
+(parse-bibtex-name "Jes{\'u}s A. {D}e Loera")
+;; #S(bibtex-name :first ((#\Space . "Jes{'u}s") (#\Space . "A.")
+;;                        (#\Space . "{D}e"))
+;;                :von nil
+;;                :last ((#\Space . "Loera"))
+;;                :jr nil)
+;;; FIXME: Original BibTeX seems to recognize a "von" token here!?!  -FIXED.
+
+(parse-bibtex-name "Jes{\'u}s A. De Loera")
+;; #S(bibtex-name :first ((#\Space . "Jes{'u}s") (#\Space . "A.")
+;;                        (#\Space . "De"))
+;;                :von nil
+;;                :last ((#\Space . "Loera"))
+;;                :jr nil)
+(parse-bibtex-name "Jes{\'u}s A. de Loera")
+;; #S(bibtex-name :first ((#\Space . "Jes{'u}s") (#\Space . "A."))
+;;                :von ((#\Space . "de"))
+;;                :last ((#\Space . "Loera"))
+;;                :jr nil)
+
+(parse-bibtex-name "{\\relax{Ch}}ristos H. Papadimitriou")
+
 ||#
+
