@@ -20,12 +20,15 @@
 	   #:*bib-entry* #:*bib-preamble* #:*bib-style*
 	   #:*bib-files* #:*cite-all-entries* #:*cite-keys*
 	   #:*bib-entry-type-functions* #:*min-crossrefs*
+	   #:bib-entry-cite-key #:bib-entry-type
 	   #:read-aux-file #:read-bib-database #:cited-bib-entries
+           #:make-bib-entry
 	   #:write-bib-entry
 	   #:read-all-bib-files-and-compute-bib-entries
 	   #:bibtex-name #:make-bibtex-name #:bibtex-name-first
 	   #:bibtex-name-von #:bibtex-name-last #:bibtex-name-jr
-	   #:*bibtex-split-initials* #:parse-bibtex-name #:parse-bibtex-name-list
+	   #:*bibtex-split-initials* #:*bibtex-split-initials-already-warned-hashtable*
+	   #:parse-bibtex-name #:parse-bibtex-name-list
 	   #:*bibtex-long-token-length* #:*bibtex-long-name-length*
 	   #:format-bibtex-name
 	   #:format-nth-bibtex-name #:num-bibtex-names #:bibtex-substring
@@ -46,3 +49,7 @@
 	   #:*bibtex-styles* #:*allow-load-lisp-bibtex-style*
 	   #:register-bibtex-style #:define-bibtex-style #:find-bibtex-style
 	   #:lisp-bibtex-style #:interpreted-bibtex-style))
+
+(defpackage :bibtex-program
+  (:use :common-lisp :bibtex-runtime :bibtex-compiler)
+  (:export #:do-emulate-bibtex #:emulate-bibtex))
